@@ -42,20 +42,22 @@
 /* This program does not delete everything it allocates since it's
    just an installer program. */
 
+#if 0
 gboolean test_only;
 gboolean verbose;
-GHashTable *included_fonts;
-gboolean debug = FALSE;
-GSList *afm_path = NULL;
-GSList *pfb_path = NULL;
-GHashTable *afm_assignments;
-GHashTable *pfb_assignments;
+#endif
+static GHashTable *included_fonts;
+static gboolean debug = FALSE;
+static GSList *afm_path = NULL;
+static GSList *pfb_path = NULL;
+static GHashTable *afm_assignments;
+static GHashTable *pfb_assignments;
 
 /* These three variables are ignored, but they are set by the command
    line arguments. */
-gboolean sys = FALSE;
-gboolean scan = FALSE;
-gboolean no_copy = FALSE;
+static gboolean sys = FALSE;
+static gboolean scan = FALSE;
+static gboolean no_copy = FALSE;
 
 typedef struct format_name
 {
@@ -63,7 +65,7 @@ typedef struct format_name
   gchar *name;
 } format_name;
 
-gint
+static gint
 g_format_name_equal (gconstpointer v, gconstpointer v2)
 {
   const format_name *a = v;
@@ -72,8 +74,7 @@ g_format_name_equal (gconstpointer v, gconstpointer v2)
 	   ( ( a->name   == a2->name   ) || ( !strcmp( a->name,   a2->name   ) ) ) );
 }
 
-/* a char* hash function from ASU */
-guint
+static guint
 g_format_name_hash (gconstpointer v)
 {
   guint h = 0;
@@ -94,64 +95,78 @@ g_format_name_hash (gconstpointer v)
 static char *
 xmlGetValue (xmlNodePtr node, const char *name)
 {
-	char *ret;
-	xmlNodePtr child;
+  char *ret;
+  xmlNodePtr child;
 
-	ret = (char *) xmlGetProp (node, name);
-	if (ret != NULL)
-		return g_strdup (ret);
-	child = node->childs;
+  ret = (char *) xmlGetProp (node, name);
+  if (ret != NULL)
+    {
+      char *gret;
+      gret = g_strdup (ret);
+      free (ret);
+      return gret;
+    }
 
-	while (child != NULL) {
-		if (!strcmp (child->name, name)) {
-		        /*
-			 * !!! Inefficient, but ...
-			 */
-			ret = xmlNodeGetContent(child);
-			if (ret != NULL)
-			    return (ret);
-		}
-		child = child->next;
+  child = node->childs;
+  while (child != NULL)
+    {
+      if (!strcmp (child->name, name))
+	{
+	  /*
+	   * !!! Inefficient, but ...
+	   */
+	  ret = xmlNodeGetContent(child);
+	  if (ret != NULL)
+	    return (ret);
 	}
+      child = child->next;
+    }
 
-	return NULL;
+  return NULL;
 }
 
 /*
  * Set a string value for a node either carried as an attibute or as
  * the content of a child.
  */
-static void xmlSetValue (xmlNodePtr node, const char *name, const char *val)
+static void
+xmlSetValue (xmlNodePtr node, const char *name, const char *val)
 {
-	const char *ret;
-	xmlNodePtr child;
+  char *ret;
+  xmlNodePtr child;
 
-	ret = xmlGetProp (node, name);
-	if (ret != NULL){
-		xmlSetProp (node, name, val);
-		return;
+  ret = xmlGetProp (node, name);
+  if (ret != NULL)
+    {
+      free (ret);
+      xmlSetProp (node, name, val);
+      return;
+    }
+
+  child = node->childs;
+  while (child != NULL)
+    {
+      if (!strcmp (child->name, name))
+	{
+	  xmlNodeSetContent (child, val);
+	  return;
 	}
-	child = node->childs;
-	while (child != NULL){
-		if (!strcmp (child->name, name)){
-			xmlNodeSetContent (child, val);
-			return;
-		}
-		child = child->next;
-	}
-	xmlSetProp (node, name, val);
+      child = child->next;
+    }
+  xmlSetProp (node, name, val);
 }
 
-gboolean
-already_included( gchar *format, gchar *name )
+static gboolean
+already_included( const gchar *format, const gchar *name )
 {
   format_name newfont;
-  newfont.format = format;
-  newfont.name = name;
+  /* The following two casts don't mean that we are going to change things.  */
+  newfont.format = (gchar *)format;
+  newfont.name = (gchar *)name;
   return g_hash_table_lookup( included_fonts, &newfont ) != NULL;
 }
 
-void
+static void
 append_font( xmlDoc *fontmap, xmlNode *font )
 {
   format_name *newfont = g_new( format_name, 1 );
@@ -163,8 +178,20 @@ append_font( xmlDoc *fontmap, xmlNode *font )
   xmlAddChild( fontmap->root, font );
 }
 
+static void
+free_font (gpointer key, gpointer value, gpointer user_data)
+{
+  format_name *font;
+
+  font = key;
+  g_free (font->format);
+  g_free (font->name);
+  g_free (key);
+}
+
+
 static gchar *
-search_path_for_file( GSList *path, gchar *file )
+search_path_for_file( GSList *path, const gchar *file )
 {
   struct stat buf;
   for ( ; path != NULL; path = path->next )
@@ -181,7 +208,7 @@ search_path_for_file( GSList *path, gchar *file )
   return NULL;
 }
 
-gint
+static gint
 check_font( xmlDoc *fontmap, xmlNode *font )
 {
   struct stat buf;
@@ -300,8 +327,8 @@ check_font( xmlDoc *fontmap, xmlNode *font )
   return ret_val;
 }
 
-void
-check_fontfile( xmlDoc *fontmap, gchar *filename )
+static void
+check_fontfile( xmlDoc *fontmap, const gchar *filename )
 {
   xmlDoc *fontfile = xmlParseFile( filename );
   if ( fontfile && fontfile->root && fontfile->root->name ) /* && fontfile->root->ns && fontfile->root->ns->href && ( ! strcmp( fontfile->root->ns->href, "http://www.gnome.org/gnome-font/0.0" ) ) ) */
@@ -323,7 +350,7 @@ check_fontfile( xmlDoc *fontmap, gchar *filename )
 	      font = next;
 	    }
 	}
-      xmlFreeDoc( fontfile );      
+      xmlFreeDoc( fontfile );
     }
   else
     {
@@ -331,13 +358,12 @@ check_fontfile( xmlDoc *fontmap, gchar *filename )
     }
 }
 
-void
-scan_fonts( xmlDoc *fontmap, gchar *path )
+static void
+scan_fonts( xmlDoc *fontmap, const gchar *path )
 {
   DIR *dir;
   struct dirent *direntry;
-  gchar *shortname;
-  
+
   dir = opendir( path );
 
   if ( dir )
@@ -346,6 +372,7 @@ scan_fonts( xmlDoc *fontmap, gchar *path )
 	{
 	  if ( strrchr( direntry->d_name, '.' ) && ! strcmp( strrchr( direntry->d_name, '.' ), ".font" ) )
 	    {
+	      const gchar *shortname;
 	      if ( strrchr( direntry->d_name, '/' ) )
 		shortname = strrchr( direntry->d_name, '/' ) + 1;
 	      else
@@ -357,7 +384,7 @@ scan_fonts( xmlDoc *fontmap, gchar *path )
 		    long_name = g_strdup_printf( "%s/%s", path, direntry->d_name );
 		  else
 		    long_name = direntry->d_name;
-	      
+
 		  check_fontfile( fontmap, long_name );
 
 		  if ( *direntry->d_name != '/' )
@@ -369,8 +396,8 @@ scan_fonts( xmlDoc *fontmap, gchar *path )
     }
 }
 
-xmlDoc *
-read_fontmap(gchar *location)
+static xmlDoc *
+read_fontmap(const gchar *location)
 {
   xmlDoc *doc = xmlParseFile( location );
   if ( doc && doc->root && doc->root->name && ( ! strcmp( doc->root->name, "fontmap" ) ) ) /* && doc->root->ns && doc->root->ns->href && ( ! strcmp( doc->root->ns->href, "http://www.gnome.org/gnome-font/0.0" ) ) ) */
@@ -393,28 +420,32 @@ read_fontmap(gchar *location)
     }
 }
 
-gint 
-write_fontmap( xmlDoc *fontmap, gchar *location )
+static gint
+write_fontmap( xmlDoc *fontmap, const gchar *location )
 {
-  return xmlSaveFile (location, fontmap);  
+  return xmlSaveFile (location, fontmap);
 }
 
 static void
-check_directory()
+check_directory(void)
 {
+  const char *dirname = DATADIR "/fonts";
+
   struct stat info;
-  gint errno;
-  if ( stat( DATADIR "/fonts", &info ) )
+  if ( stat( dirname, &info ) )
     {
       switch ( errno )
 	{
 	case ENOENT:
-	  if ( mkdir( DATADIR "/fonts", 0777 ) )
+	  if ( mkdir( dirname, 0777 ) )
 	    {
 	      switch( errno )
 		{
+		case EEXIST:
+		  /* Someone beat us to it.  */
+		  break;
 		default:
-		  g_error( "Unable to create directory %s.", DATADIR "/fonts" );
+		  g_error( "Unable to create directory %s.", dirname );
 		  break;
 		}
 	    }
@@ -441,11 +472,13 @@ add_assignment(poptContext ctx,
 	{
 	  *comma = 0;
 	  path = g_strdup( comma + 1 );
-	  key = g_strdup( (char *) arg );
 	  if ( !stat( path, &info ) )
 	    {
+	      key = g_strdup( (char *) arg );
 	      g_hash_table_insert( afm_assignments, key, g_slist_append( g_hash_table_lookup( afm_assignments, key ), path ) );
 	    }
+	  else
+	    g_free (path);
 	}
     }
   else if ( opt->shortName == 'P' )
@@ -459,17 +492,32 @@ add_assignment(poptContext ctx,
 	{
 	  *comma = 0;
 	  path = g_strdup( comma + 1 );
-	  key = g_strdup( (char *) arg );
 	  if ( !stat( path, &info ) )
 	    {
+	      key = g_strdup( (char *) arg );
 	      g_hash_table_insert( pfb_assignments, key, g_slist_append( g_hash_table_lookup( pfb_assignments, key ), path ) );
 	    }
+	  else
+	    g_free (path);
+
 	  *comma = ',';
 	}
     }
   /* else something's weird :) */
 }
 
+static void
+free_assignment (gpointer key, gpointer value, gpointer user_data)
+{
+  GSList *path, *tmp;
+
+  g_free (key);
+
+  for (tmp = path = value; tmp; tmp = tmp->next)
+    g_free (tmp->data);
+  g_slist_free (path);
+}
+
 static void
 add_path(poptContext ctx,
 	 enum poptCallbackReason reason,
@@ -518,34 +566,44 @@ static const struct poptOption options[] = {
   { NULL, '\0', 0, NULL, 0 }
 };
 
-poptContext ctx;
+static poptContext ctx;
 
 static void
 init( gint argc, gchar *argv[] )
 {
-  char *appdesc;
   int flags = 0;
-  
+
   gnomelib_init ("gnome-font-install", VERSION);
-  
+
   if(options)
     {
-      appdesc = "gnome-font-install options";
+      const char *appdesc = "gnome-font-install options";
       gnomelib_register_popt_table(options, appdesc);
     }
-  
+
   ctx = gnomelib_parse_args(argc, argv, flags);
-  
+
   /* reduce mem usage (hopefully) */
   gnome_config_sync();
   g_blow_chunks();
 }
 
+void
+warn_missing_font (const char *font_name)
+{
+  g_warning ("Warning: the %s font was not found. Printing is unlikely\n"
+	     "to work unless basic fonts are installed. Please see:\n"
+	     "\n"
+	     "http://www.levien.com/gnome/font-install.html\n"
+	     "\n"
+	     "for more information.", font_name);
+}
+
 gint
 main( gint argc, gchar *argv[])
 {
   xmlDoc *fontmap;
-  gchar *fontmap_loc = DATADIR "/fonts/fontmap";
+  const gchar *fontmap_loc = DATADIR "/fonts/fontmap";
   gint i;
   gboolean scan_dot = TRUE;
   char **args;
@@ -561,16 +619,17 @@ main( gint argc, gchar *argv[])
   init( argc, argv );
 
   check_directory();
-  
+
   included_fonts = g_hash_table_new( g_format_name_hash,
 				     g_format_name_equal );
 
   /* I want this to overwrite the old data until the .font files are better reviewed. */
-#if 0
-  fontmap = read_fontmap( fontmap_loc );
+  if (0)
+    fontmap = read_fontmap( fontmap_loc );
+  else
+    fontmap = NULL;
   if ( fontmap == NULL )
     {
-#endif
       fontmap = xmlNewDoc( "1.0" );
       if ( fontmap == NULL )
 	{
@@ -578,9 +637,7 @@ main( gint argc, gchar *argv[])
 	  return -1;
 	}
       fontmap->root = xmlNewDocNode( fontmap, NULL, "fontmap", NULL );
-#if 0
     }
-#endif
 
   args = poptGetArgs(ctx);
   for(i = 0; args && args[i]; i++)
@@ -591,7 +648,25 @@ main( gint argc, gchar *argv[])
   if ( scan_dot )
     scan_fonts( fontmap, "." );
   write_fontmap( fontmap, fontmap_loc );
-  
+
   poptFreeContext(ctx);
+  xmlFreeDoc (fontmap);
+
+  if (!already_included ("type1", "Times-Roman"))
+    warn_missing_font ("Times");
+  else if (!already_included ("type1", "Helvetica"))
+    warn_missing_font ("Helvetica");
+  else if (!already_included ("type1", "Courier"))
+    warn_missing_font ("Courier");
+
+  g_hash_table_foreach (afm_assignments, free_assignment, NULL);
+  g_hash_table_destroy (afm_assignments);
+
+  g_hash_table_foreach (pfb_assignments, free_assignment, NULL);
+  g_hash_table_destroy (pfb_assignments);
+
+  g_hash_table_foreach (included_fonts, free_font, NULL);
+  g_hash_table_destroy (included_fonts);
+
   return 0;
 }
