@@ -26,6 +26,16 @@ print_test_page (GnomePrintContext *pc)
   gnome_print_gsave (pc);
   gnome_print_concat (pc, matrix);
   font = gnome_font_new_closest ("Times", GNOME_FONT_BOLD, 1, 36);
+  if (font == NULL)
+    {
+      g_warning (
+"Since Times cannot be found, it would appear that fonts have\n"
+"not been correctly installed. If you have Adobe or URW .pfb font files,\n"
+"you can try:\n\n"
+"./gnome-font-install --system --scan --no-copy --afm-path=%s/fonts/afms --pfb-assignment=ghostscript,<path> fonts/\n\n"
+"Or see http://www.levien.com/gnome/font-install.html", DATADIR);
+      exit (0);
+    }
   gnome_print_setfont (pc, font);
   gnome_print_setrgbcolor (pc, 0.9, 0.8, 0.6);
   width = gnome_font_get_width_string (font, _("Gnome-print test page"));
